/* Base error class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_error.h,v 1.8 2001/10/10 13:39:39 glass Exp $
 */

/**@#-*/
#ifndef _PM_ERROR_H
#define _PM_ERROR_H

#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "pm_noncopyable.h"
/**@#+*/

/**
 * Base error class.
 *
 * The PMError class is the base class from which all
 * exception objects thrown in the library are derived.
 * These classes retrieve error information and text that
 * you can subsequently use to create an exception object.
 * <p>
 * None of the functions in this class throws exceptions because
 * an exception probably has been thrown already or is about to be
 * thrown.
 *
 * @author Dmitry A Steklenev
 */

class PMError
{
  public:
    /** Constructs error from current GUI error. */
    PMError( const char* file, const char* func, int line );
    /** Constructs error from specified error id and information. */
    PMError( int code, const char* info,
                       const char* file, const char* func, int line );

    /** Constructs error object from another error. */
    PMError( const PMError& );
    /** Assigns the value of one error object to another. */
    PMError& operator=( const PMError& );
    /** Destructs the error object */
   ~PMError();

    /** Returns source file name. */
    const char* file() const { return err_file; }
    /** Returns the name of the function. */
    const char* func() const { return err_func; }
    /** Returns the line number. */
    int         line() const { return err_line; }
    /** Returns the error message. */
    const char* info() const { return err_info; }
    /** Returns the error id. */
    int         code() const { return err_code; }

    /** Display error information. */
    PMError& display( FILE* out = 0 );

  private:
    char* err_file;
    char* err_func;
    int   err_line;
    char* err_info;
    int   err_code;
};

/**@#-*/
#define PM_ERROR_LOCATION  __FILE__, __FUNCTION__, __LINE__

#define PM_THROW_ERROR( id, info ) \
        throw( PMError( id, info, PM_ERROR_LOCATION ).display( stdout ))

#define PM_THROW_GUIERROR()   throw( PMError( PM_ERROR_LOCATION ).display( stdout ))
#define PM_THROW_CLBERROR()   PM_THROW_ERROR( errno  , strerror( errno ))
#define PM_THROW_OS2ERROR(rc) PM_THROW_ERROR( rc     , NULL )

#endif
