/* Serialize access to resources
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_exclusion.h,v 1.4 2001/10/31 10:01:04 glass Exp $
 */

/**@#-*/
#ifndef _PM_EXCLUSION_H
#define _PM_EXCLUSION_H

#include "pm_noncopyable.h"
/**@#+*/

/**
 * Serialize access to resources.
 *
 * A mutual exclusion class PMExclusion protects resources
 * (such as files, data in memory, and peripheral devices) from
 * simultaneous access by several processes. Class also enable threads
 * to serialize their access to resources.
 *
 * None of the functions in this class throws exceptions because
 * an exception probably has been thrown already or is about
 * to be thrown.
 *
 * @author Dmitry A Steklenev
 */

class PMExclusion : public PMNonCopyable
{
  public:
    /** Constructs the exclusion object. */
    PMExclusion();
    /** Destructs the exclusion object. */
   ~PMExclusion();

    /**
     * Request access to resource.
     *
     * Requests ownership of a resource.
     * Blocks the calling thread indefinitely.
     *
     * @return true, if ownership established.
     */

    BOOL request();

    /**
     * Request access to resource with wait timeout.
     *
     * Requests ownership of a resource.
     * Blocks the calling thread.
     *
     * @param mseq this is the maximum amount of time the
     *             user wants to allow the thread to be blocked.
     *
     * @return true, if ownership established.
     */

    BOOL request( unsigned long msec );

    /**
     * Relinquishes ownership of a resource that was requested by
     * <i>request</i>.
     *
     * Only the thread that owns the resource can issue release().
     *
     * @return true, if ownership relinquished.
     */

    BOOL release();

  private:
    HMTX exc_handle;
};

/**@#-*/
#endif
