/* Prohibit copy construction and copy assignment
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_noncopyable.h,v 1.2 2001/10/10 13:39:39 glass Exp $
 */

/**@#-*/
#ifndef _PM_NONCOPYABLE_H
#define _PM_NONCOPYABLE_H
/**@#+*/

/**
 * Prohibit copy construction and copy assignment.
 *
 * Class noncopyable is a base class. Derive your own class from
 * noncopyable when you want to prohibit copy construction and
 * copy assignment.
 * <p>
 * Some objects, particularly those which hold complex resources like
 * files or network connections, have no sensible copy semantics.
 * Sometimes there are possible copy semantics, but these would be
 * of very limited usefulness and be very difficult to implement
 * correctly.  Sometimes you're implementing a class that doesn't
 * need to be copied just yet and you don't want to take the time
 * to write the appropriate functions. Deriving from noncopyable
 * will prevent the otherwise implicitly-generated functions
 * (which don't have the proper semantics) from becoming a
 * trap for other programmers.
 * <p>
 * The traditional way to deal with these is to declare a private
 * copy constructor and copy assignment, and then document why this
 * is done. But deriving from noncopyable is simpler and
 * clearer, and doesn't require additional documentation.
 *
 * @author Dmitry A Steklenev
 */

class PMNonCopyable
{
  public:
    PMNonCopyable() {}
  private:
    PMNonCopyable( const PMNonCopyable& );
    PMNonCopyable& operator=( const PMNonCopyable& );
};

/**@#-*/
#endif
