/* Thread class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_thread.h,v 1.8 2002/09/12 08:36:58 glass Exp $
 */

/**@#-*/
#ifndef _PM_THREAD_H
#define _PM_THREAD_H

#include "pm_noncopyable.h"
/**@#+*/

/**
 * Thread class.
 *
 * The PMThread class represents threads of execution within
 * the current program. You can use this class to implement
 * multithreaded applications. Objects of this class give you
 * access to all of the tasking APIs of the operating system.
 * In addition, these objects can serve as the anchor for thread-specific
 * information.
 *
 * @author Dmitry A Steklenev
 */

class PMThread : public PMNonCopyable {

  public:

    /**
     * Constructs the thread object.
     *
     * Use this, the default constructor, to create an object of this
     * class without creating an additional thread of execution.
     * You can subsequently start execution of the thread using
     * PMThread::start.
     */

    PMThread();

    /**
     * Obtains the identifier of the thread.
     * @return
     *   A return value of -1 indicates that the thread is
     *   not started.
     */

    TID tid() const;

    /** Determines if this thread is currently active. */
    BOOL is_alive() const;
    /** This function causes the thread to start immediately. */
    virtual PMThread& start();

    /**
     * Waits the thread.
     *
     * Places the current thread into a wait state until thread,
     * represented of the PMThread object, has ended.
     */

    virtual PMThread& join();

    /**
     * Suspends the thread.
     *
     * Suspends the thread's execution, which can be undone later
     * with a call to resume(). Note that suspend() and resume() are
     * not recommended mechanisms for thread control. Resources and
     * conditions provide safer services for managing threads.
     */

    virtual PMThread& suspend();

    /**
     * Resumes the thread.
     *
     * Resumes a thread's execution after it has been suspended by a
     * call to suspend(). Note that suspend() and resume() are not
     * recommended mechanisms for thread control. Resources and
     * conditions provide safer services for managing threads.
     */

    virtual PMThread& resume();

    /**
     * Stops the thread.
     *
     * This function causes the thread to stop immediately, bypassing
     * some normal termination functions. Use it with extreme caution,
     * preferably when the state of the target thread is known.
     */

    virtual PMThread& stop();

  protected:

    /**
     * Runable interface.
     *
     * You must overwrite this metod.
     */

    virtual void run() = 0;

  private:
    /** Thread identifier */ TID  run_id;
    /** Is thread alive   */ BOOL run_alive;

    static void launch( PMThread* );
};

#if !defined(__ccdoc__)
/*--------------------------------------------------
 * Obtains the identifier of the thread
 *-------------------------------------------------*/
inline TID PMThread::tid() const
{
  return run_id;
}

/*--------------------------------------------------
 * Determines if this thread is currently active
 *-------------------------------------------------*/
inline BOOL PMThread::is_alive() const
{
  return run_alive;
}
#endif
/**@#-*/
#endif
